using MPI, Optim, LineSearches, StellaratorOptimization, StellaratorOptimizationMetrics, VMEC
#ENV["JULIA_DEBUG"] = StellaratorOptimization
if !MPI.Initialized()
  MPI.Init()
end

comms = StellOpt.setup_comms(8, MPI.COMM_WORLD)

optimizer = StellOpt.Optimizer(ConjugateGradient(; alphaguess=1e-5, linesearch=BackTracking()),
                               Optim.Options(iterations = 1))
prob = StellOpt.Problem()
prob.output_directory = "/home/bfaber/data/opt/lassoruns/spsa_vmec_test/"
StellOpt.add_optimizer!(prob, optimizer)

fd = StellOpt.ForwardDiff(1e-4)
spsa = StellOpt.SPSA(δ = 1e-4)
#nml = NE3DLE.read_par_file("/home/bfaber/.julia/dev/NE3DLE/NE3DLE.in")
nml = VMEC.read_vmec_namelist("/home/bfaber/projects/julia/StellaratorOptimization/input.aten")
vmec = readVmecWout("/home/bfaber/.julia/dev/VMEC/test/wout_aten.nc")

vmec_wrapper = StellOpt.EquilibriumWrapper(nml, Vmec, spsa)
StellOpt.update_equilibrium!(vmec_wrapper, vmec)
StellOpt.add_equilibrium_wrapper!(prob, vmec_wrapper)

vmec_s05 = StellOpt.@add_derived_geometry!(prob, VmecSurface(0.5, equilibrium=vmec_wrapper))

StellOpt.@add_target!(prob, VMEC.quasisymmetry_deviation(1, 4, 16, 16, vmec_s05), 0.0, 1.0, "qs")
ζ_range = 0.0:2π/128:100π
B_resolution = 100
StellOpt.@add_mpi_target!(prob,
                          StellOptMetrics.GammaC.gamma_c_target(vmec_s05, first(ζ_range), step(ζ_range), last(ζ_range),
                                                                B_resolution, comms, false, true), 0.0, 1.0, "Γc(s = 0.5)")
#StellOpt.@add_target!(prob, VMEC.magnetic_well(vmec_s05), -0.1, 1.0, "well")

StellOpt.add_variable!(prob, vmec_wrapper, "rbc(1,0)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(1,0)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(2,0)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(2,0)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(3,0)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(3,0)")
#=
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(-3,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(-3,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(-2,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(-2,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(-1,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(-1,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(0,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(0,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(1,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(1,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(2,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(2,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "rbc(3,1)")
StellOpt.add_variable!(prob, vmec_wrapper, "zbs(3,1)")
=#
#=
for k in keys(prob.targets)
    prob.targets[k].gradient = zeros(Float64, length(prob.state_vector))
end
if comms.ranks.world == 0
    StellOpt.objective_gradient!(0.0, zeros(Float64, length(prob.state_vector)), StellOpt.state_vector_values(prob), prob, comms)
    MPI.bcast(true, 0, comms.world_comm)
else
    StellOpt.worker_loop(prob, comms)
end
=#
res = StellOpt.optimize_mpi(prob, comms)
if comms.ranks.world == 0
    for t in values(prob.targets)
        @info "Target $(t.label): $(t.current_value), $(t.gradient)"
    end
end
